@@ -3,7 +3,7 @@
 ;;;     Title: The X Render Extension
 ;;;   Created: 2002-08-03
 ;;;    Author: Gilbert Baumann <unk6@rz.uni-karlsruhe.de>
-;;;       xrender.lisp,v 1.5 2004/12/06 11:48:57 csr21 Exp
+;;;       $Id: xrender.lisp,v 1.5 2004/12/06 11:48:57 csr21 Exp $
 ;;; ---------------------------------------------------------------------------
 ;;;
 ;;; (c) copyright 2002, 2003 by Gilbert Baumann
@@ -127,17 +127,15 @@
           render-query-version
           ;; render-query-picture-formats
           render-fill-rectangle
+          render-triangles
+          render-trapezoids
           render-composite
           render-create-glyph-set
           render-reference-glyph-set
           render-composite-glyphs
           render-add-glyph
           render-add-glyph-from-picture
-          render-free-glyphs
-          
-          
-	  render-combine
-	  ))
+          render-free-glyphs))
 
 (pushnew :clx-ext-render *features*)
 
@@ -199,6 +197,24 @@
 ;; We do away with the distinction between pict-format and
 ;; picture-format-info. That is we cache picture-format-infos.
 
+(defstruct picture-format
+  display 
+  (id   0 :type (unsigned-byte 29))
+  type
+  depth
+  red-byte
+  green-byte
+  blue-byte
+  alpha-byte
+  colormap)
+
+(def-clx-class (glyph-set (:copier nil)
+                        )
+  (id 0 :type resource-id)
+  (display nil :type (or null display))
+  (plist nil :type list)                ; Extension hook
+  (format))
+
 (defstruct render-info
   major-version
   minor-version
@@ -269,8 +285,7 @@ by every function, which attempts to generate RENDER requests."
 
 (defun find-window-picture-format (window)
   "Find the picture format which matches the given window."
-  (let* ((cm (window-colormap window))
-         (vi (colormap-visual-info cm))
+  (let* ((vi (window-visual-info window))
          (display (window-display window)))
     (ensure-render-initialized display)
     (case (visual-info-class vi)
@@ -302,17 +317,6 @@ by every function, which attempts to generate RENDER requests."
 
 ;;; picture format
 
-(defstruct picture-format
-  display 
-  (id   0 :type (unsigned-byte 29))
-  type
-  depth
-  red-byte
-  green-byte
-  blue-byte
-  alpha-byte
-  colormap)
-
 (defmethod print-object ((object picture-format) stream)
   (let ((abbrev
          (with-output-to-string (bag)
@@ -341,10 +345,43 @@ by every function, which attempts to generate RENDER requests."
   (define-accessor render-op (8)
     ((index) `(member8-get ,index
                :clear :src :dst :over :over-reverse :in :in-reverse
-               :out :out-reverse :atop :atop-reverse :xor :add :saturate :maximum))
+               :out :out-reverse :atop :atop-reverse :xor :add :saturate
+               '#:undefined-pict-op-Eh '#:undefined-pict-op-Fh
+               :disjoint-clear :disjoint-src :disjoint-dst :disjoint-over
+               :disjoint-over-reverse :disjoint-in :disjoint-in-reverse
+               :disjoint-out :disjoint-out-reverse :disjoint-atop
+               :disjoint-atop-reverse :disjoint-xor
+               '#:undefined-pict-op-1Ch '#:undefined-pict-op-1Dh
+               '#:undefined-pict-op-1Eh '#:undefined-pict-op-1Fh
+               :conjoint-clear :conjoint-src :conjoint-dst :conjoint-over
+               :conjoint-over-reverse :conjoint-in :conjoint-in-reverse
+               :conjoint-out :conjoint-out-reverse :conjoint-atop
+               :conjoint-atop-reverse :conjoint-xor))
     ((index thing) `(member8-put ,index ,thing
                      :clear :src :dst :over :over-reverse :in :in-reverse
-                     :out :out-reverse :atop :atop-reverse :xor :add :saturate :maximum))))
+                     :out :out-reverse :atop :atop-reverse :xor :add :saturate
+                     '#:undefined-pict-op-Eh '#:undefined-pict-op-Fh
+                     :disjoint-clear :disjoint-src :disjoint-dst :disjoint-over
+                     :disjoint-over-reverse :disjoint-in :disjoint-in-reverse
+                     :disjoint-out :disjoint-out-reverse :disjoint-atop
+                     :disjoint-atop-reverse :disjoint-xor
+                     '#:undefined-pict-op-1Ch '#:undefined-pict-op-1Dh
+                     '#:undefined-pict-op-1Eh '#:undefined-pict-op-1Fh
+                     :conjoint-clear :conjoint-src :conjoint-dst :conjoint-over
+                     :conjoint-over-reverse :conjoint-in :conjoint-in-reverse
+                     :conjoint-out :conjoint-out-reverse :conjoint-atop
+                     :conjoint-atop-reverse :conjoint-xor)))
+  (deftype render-op ()
+    '(member :clear :src :dst :over :over-reverse :in :in-reverse
+      :out :out-reverse :atop :atop-reverse :xor :add :saturate
+      :disjoint-clear :disjoint-src :disjoint-dst :disjoint-over
+      :disjoint-over-reverse :disjoint-in :disjoint-in-reverse
+      :disjoint-out :disjoint-out-reverse :disjoint-atop
+      :disjoint-atop-reverse :disjoint-xor
+      :conjoint-clear :conjoint-src :conjoint-dst :conjoint-over
+      :conjoint-over-reverse :conjoint-in :conjoint-in-reverse
+      :conjoint-out :conjoint-out-reverse :conjoint-atop
+      :conjoint-atop-reverse :conjoint-xor)))
 
 ;; Now these pictures objects are like graphics contexts. I was about
 ;; to introduce a synchronous mode, realizing that the RENDER protocol
@@ -417,9 +454,9 @@ by every function, which attempts to generate RENDER requests."
                                     ,index))
                            (%render-change-picture-clip-rectangles
                             picture (aref (picture-%values picture) ,index))
-                           (setf (aref (picture-%values picture) ,index)
-                            (aref (picture-%server-values picture)
-                             ,index))))
+                           (setf (aref (picture-%server-values picture) ,index)
+                                 (aref (picture-%values picture) ,index))))
+
                    (setf (picture-%changed-p picture) nil)))
 
                (defun render-create-picture
@@ -541,16 +578,16 @@ by every function, which attempts to generate RENDER requests."
     (synchronise-picture-state picture)
     (with-buffer-request (display (extension-opcode display "RENDER"))
       (data +X-RenderFillRectangles+)
-      (render-op op)                         ;op
-      (card8 0)                        ;pad
-      (card16 0)                        ;pad
+      (render-op op)
+      (pad8 0)
+      (pad16 0)
       (resource-id (picture-id picture))
       (card16 (elt color 0)) (card16 (elt color 1)) (card16 (elt color 2)) (card16 (elt color 3))
       (int16 x1) (int16 y1) (card16 w) (card16 h))))
 
 ;; fill rectangles, colors.
 
-(defun render-triangles-1 (picture op source src-x src-y format coord-sequence)
+(defun render-triangles (picture op source src-x src-y format coord-sequence)
   ;; For performance reasons we do a special typecase on (simple-array
   ;; (unsigned-byte 32) (*)), so that it'll be possible to have high
   ;; performance rasters.
@@ -558,17 +595,18 @@ by every function, which attempts to generate RENDER requests."
                '(let ((display (picture-display picture)))
                  (synchronise-picture-state picture)
                  (synchronise-picture-state source)
-                 (with-buffer-request (display (extension-opcode display "RENDER"))
-                   (data +X-RenderTriangles+)
-                   (render-op op)                       ;op
-                   (card8 0)                            ;pad
-                   (card16 0)                           ;pad
-                   (resource-id (picture-id source))
-                   (resource-id (picture-id picture))
-                   (picture-format format)
-                   (int16 src-x)
-                   (int16 src-y)
-                   ((sequence :format int32) coord-sequence) ))))
+                 (labels ((funk (x) (ash x 16)))
+                   (with-buffer-request (display (extension-opcode display "RENDER"))
+                     (data +X-RenderTriangles+)
+                     (render-op op)
+                     (pad8 0)
+                     (pad16 0)
+                     (resource-id (picture-id source))
+                     (resource-id (picture-id picture))
+                     (picture-format format)
+                     (int16 src-x)
+                     (int16 src-y)
+                     ((sequence :format int32 :transform #'funk) coord-sequence))))))
     (typecase coord-sequence
       ((simple-array (unsigned-byte 32) (*))
        (locally
@@ -665,7 +703,7 @@ by every function, which attempts to generate RENDER requests."
       (data +X-RenderSetPictureFilter+)
       (resource-id (picture-id picture))
       (card16 (length filter))
-      (card16 0)                        ;pad
+      (pad16 0)
       ((sequence :format card8) (map 'vector #'char-code filter)))))
   
 
@@ -676,25 +714,26 @@ by every function, which attempts to generate RENDER requests."
   )
 ||#
 
-(defun render-trapezoids-1 (picture op source src-x src-y format coord-sequence)
+(defun render-trapezoids (picture op source src-x src-y mask-format coord-sequence)
   ;; coord-sequence is  top bottom
-  ;;                    line-1-x1 line-1-y1 line-1-x2 line-1-y2
-  ;;                    line-2-x1 line-2-y1 line-2-x2 line-2-y2 ...
+  ;;                    left-x1 left-y1 left-x2 left-y2
+  ;;                    right-x1 right-y1 right-x2 right-y2 ...
   ;;
   (let ((display (picture-display picture)))
     (synchronise-picture-state picture)
     (synchronise-picture-state source)
-    (with-buffer-request (display (extension-opcode display "RENDER"))
-      (data +X-RenderTrapezoids+)
-      (render-op op)                    ;op
-      (card8 0)                         ;pad
-      (card16 0)                        ;pad
-      (resource-id (picture-id source))
-      (resource-id (picture-id picture))
-      (picture-format format)
-      (int16 src-x)
-      (int16 src-y)
-      ((sequence :format int32) coord-sequence) )))
+    (labels ((funk (x) (ash x 16)))
+      (with-buffer-request (display (extension-opcode display "RENDER"))
+        (data +X-RenderTrapezoids+)
+        (render-op op)
+        (pad8 0)
+        (pad16 0)
+        (resource-id (picture-id source))
+        (resource-id (picture-id picture))
+        ((or (member :none) picture-format) mask-format)
+        (int16 src-x)
+        (int16 src-y)
+        ((sequence :format int32 :transform #'funk) coord-sequence)))))
 
 (defun render-composite (op
                          source mask dest
@@ -706,28 +745,21 @@ by every function, which attempts to generate RENDER requests."
     (synchronise-picture-state dest)
     (with-buffer-request (display (extension-opcode display "RENDER"))
       (data +X-RenderComposite+)
-      (render-op op)                         ;op
-      (card8 0)                         ;pad
-      (card16 0)                        ;pad
+      (render-op op)
+      (pad8 0)
+      (pad16 0)
       (resource-id (picture-id source))
       (resource-id (if mask (picture-id mask) 0))
       (resource-id (picture-id dest))
-      (card16 src-x)
-      (card16 src-y)
-      (card16 mask-x)
-      (card16 mask-y)
-      (card16 dst-x)
-      (card16 dst-y)
+      (int16 src-x)
+      (int16 src-y)
+      (int16 mask-x)
+      (int16 mask-y)
+      (int16 dst-x)
+      (int16 dst-y)
       (card16 width)
       (card16 height))))
 
-(def-clx-class (glyph-set (:copier nil)
-                        )
-  (id 0 :type resource-id)
-  (display nil :type (or null display))
-  (plist nil :type list)                ; Extension hook
-  (format))
-
 (defun render-create-glyph-set (format &key glyph-set)
   (let ((display (picture-format-display format)))
     (let* ((glyph-set (or glyph-set (make-glyph-set :display display)))
@@ -774,33 +806,74 @@ by every function, which attempts to generate RENDER requests."
     (with-buffer-request (display (extension-opcode display "RENDER"))
       (data +X-RenderCompositeGlyphs8+)
       (render-op alu)
-      (card8 0) (card16 0)              ;padding
+      (pad8 0)
+      (pad16 0)
       (picture source)
       (picture dest)
       ((or (member :none) picture-format) mask-format)
       (glyph-set glyph-set)
       (int16 src-x) (int16 src-y)
       (card8 (- end start)) ;length of glyph elt
-      (card8 0) (card16 0) ;padding
-      (card16 dest-x) (card16 dest-y)             ;dx, dy
+      (pad8 0)
+      (pad16 0)
+      (int16 dest-x) (int16 dest-y)             ;dx, dy
       ((sequence :format card8) sequence))))
 
 (defmacro %render-composite-glyphs
-    (opcode type transform
-          display dest glyph-set source dest-x dest-y sequence alu src-x src-y mask-format start end)
-  `(with-buffer-request (,display (extension-opcode ,display "RENDER"))
-    (data ,opcode)
-    (render-op ,alu)
-    (card8 0) (card16 0)                        ;padding
-    (picture ,source)
-    (picture ,dest)
-    ((or (member :none) picture-format) ,mask-format)
-    (glyph-set ,glyph-set)
-    (int16 ,src-x) (int16 ,src-y)
-    (card8 (- ,end ,start))                     ;length of glyph elt
-    (card8 0) (card16 0)                        ;padding? really?
-    (card16 ,dest-x) (card16 ,dest-y)           ;dx, dy
-    ((sequence :format ,type :start ,start :end ,end :transform ,transform) ,sequence)))
+    (opcode type transform display dest glyph-set source dest-x dest-y sequence
+     alu src-x src-y mask-format start end)
+  (let ((size (ecase type (card8 1) (card16 2) (card32 4)))
+	;; FIXME: the last chunk for CARD8 can be 254.
+	(chunksize (ecase type (card8 252) (card16 254) (card32 254))))
+    `(multiple-value-bind (nchunks leftover)
+         (floor (- end start) ,chunksize)
+       (let* ((payloadsize (+ (* nchunks (+ 8 (* ,chunksize ,size)))
+			      (if (> leftover 0)
+				  (+ 8 (* 4 (ceiling (* leftover ,size) 4)))
+				  0)))
+	      (request-length (+ 7 (/ payloadsize 4))))
+	 (declare (integer request-length))
+	 (with-buffer-request (,display (extension-opcode ,display "RENDER") :length (* 4 request-length))
+	   (data ,opcode)
+	   (length request-length)
+	   (render-op ,alu)
+           (pad8 0)
+	   (pad16 0)
+	   (picture ,source)
+	   (picture ,dest)
+	   ((or (member :none) picture-format) ,mask-format)
+	   (glyph-set ,glyph-set)
+	   (int16 ,src-x) (int16 ,src-y)
+	   (progn
+	     (let ((boffset (+ buffer-boffset 28))
+		   (start ,start)
+		   (end ,end)
+		   (dest-x ,dest-x)
+		   (dest-y ,dest-y))
+	       (dotimes (i nchunks)
+		 (set-buffer-offset boffset)
+		 (put-items (0)
+		   (card8 ,chunksize)
+		   (card8 0)
+		   (card16 0)
+		   (int16 dest-x)
+		   (int16 dest-y)
+		   ((sequence :start start :end (+ start ,chunksize) :format ,type :transform ,transform :appending t) ,sequence))
+		 (setq dest-x 0 dest-y 0)
+		 (incf boffset (+ 8 (* ,chunksize ,size)))
+		 (incf start ,chunksize))
+	       (when (> leftover 0)
+		 (set-buffer-offset boffset)
+		 (put-items (0)
+		   (card8 leftover)
+		   (card8 0)
+		   (card16 0)
+		   (int16 dest-x)
+		   (int16 dest-y)
+		   ((sequence :start start :end end :format ,type :transform ,transform :appending t) ,sequence))
+		 ;; padding?
+		 (incf boffset (+ 8 (* 4 (ceiling (* leftover ,size) 4)))))
+	       (setf (buffer-boffset ,display) boffset))))))))
 
 (defun render-composite-glyphs (dest glyph-set source dest-x dest-y sequence
                                 &key (op :over)
@@ -998,17 +1071,17 @@ by every function, which attempts to generate RENDER requests."
            (render-fill-rectangle px.pic :src
                                   (list #x8000 #x0000 #x8000 #xFFFF)
                                   0 0 256 256)
-           ;; render-combine simply does not work
-           (render-combine :src pic pic px.pic
-                           350 350 350 350 0 0 256 256)
+
+           (render-composite :src pic pic px.pic
+                             350 350 350 350 0 0 256 256)
            ;;
            (render-fill-rectangle px.pic :over
                                   (list #x8000 #x8000 #x8000 #x8000)
                                   0 0 100 100)
-           (render-combine :src
-                           px.pic px.pic pic
-                           0 0 0 0 350 350
-                           256 256)
+           (render-composite :src
+                             px.pic px.pic pic
+                             0 0 0 0 350 350
+                             256 256)
            (render-fill-rectangle pic op (list #x0 #x0 #x0 #x8000) 200 200 800 800)
            (display-finish-output dpy))
       (close-display dpy))))
@@ -1037,23 +1110,22 @@ by every function, which attempts to generate RENDER requests."
                (xlib:draw-point px px.gc x y)
                ))
            (xlib:clear-area win)
-           (let ((q(render-create-picture px
-                                          :format
-                                          (first (find-matching-picture-formats
-                                                  dpy
-                                                  :depth 32
-                                                  :alpha 8 :red 8 :green 8 :blue 8))
-                                          :component-alpha :on
-                                          :repeat :off
-                                          )))
-             (render-combine op
-                             q
-                             q 
-                             pic
-                             0 0
-                             0 0
-                             100 100
-                             400 400))
+           (let ((q (render-create-picture px
+                                           :format
+                                           (first (find-matching-picture-formats
+                                                   dpy
+                                                   :depth 32
+                                                   :alpha 8 :red 8 :green 8 :blue 8))
+                                           :component-alpha :on
+                                           :repeat :off)))
+             (render-composite op
+                               q
+                               q 
+                               pic
+                               0 0
+                               0 0
+                               100 100
+                               400 400))
            (let ()
              ;;(render-fill-rectangle pic op (list 255 255 255 255) 100 100 200 200)
              (display-finish-output dpy)))
@@ -1087,3 +1159,21 @@ by every function, which attempts to generate RENDER requests."
         (card16 x)
         (card16 y))
       cursor)))
+
+(defun render-create-anim-cursor (cursors delays)
+  "Create animated cursor. cursors length must be the same as delays length."
+  (let ((display (cursor-display (first cursors))))
+    (ensure-render-initialized display)
+    (let* ((cursor (make-cursor :display display))
+           (cid (allocate-resource-id display cursor 'cursor))
+           (cursors-length (length cursors))
+           (cursors-delays (make-list (* 2 (length cursors)))))
+      (setf (xlib:cursor-id cursor) cid)
+      (dotimes (i cursors-length)
+        (setf (elt cursors-delays (* 2 i)) (cursor-id (elt cursors i))
+              (elt cursors-delays (1+ (* 2 i))) (elt delays i)))
+      (xlib::with-buffer-request (display (extension-opcode display "RENDER"))
+        (data +X-RenderCreateAnimCursor+)
+        (resource-id cid)
+        ((sequence :format card32) cursors-delays))
+      cursor)))
